/*
 The software in this package is distributed under the GNU General
 Public License version 2 (with a special exception described below).

 A copy of GNU General Public License (GPL) is included in this distribution,
 in the file COPYING.GPL.

 As a special exception, if other files instantiate templates or use macros
 or inline functions from this file, or you compile this file and link it
 with other works to produce a work based on this file, this file
 does not by itself cause the resulting work to be covered
 by the GNU General Public License.

 However the source code for this file must still be made available
 in accordance with section (3) of the GNU General Public License.

 This exception does not invalidate any other reasons why a work based
 on this file might be covered by the GNU General Public License.

 Christian Herdtweck, Intra2net AG 2015
 Based on an example in Boost Documentation (by Christopher M. Kohlhoff)
 and adaptation by Guilherme M. Ferreira
 */

#include "icmp/icmpdestinationunreachabledata.h"

#include <logfunc.hpp>
using I2n::Logger::GlobalLogger;


IcmpDestinationUnreachableData::IcmpDestinationUnreachableData(
                                                     const std::size_t size_arg)
    : IcmpData_PingFailReply( size_arg )
{}

bool IcmpDestinationUnreachableData::match_destination_unreachable(
                                           const uint16_t identifier,
                                           const uint16_t sequence_number) const
{
    return IcmpData_PingFailReply::match_ping_request(identifier,
                                                      sequence_number);
}

bool IcmpDestinationUnreachableData::match_time_exceeded(
                                           const uint16_t identifier,
                                           const uint16_t sequence_number) const
{ return false; }

/**
 * @brief Prints the destination unreachable messages.
 *
 * @return void
 */
void IcmpDestinationUnreachableData::print(
            const size_t &bytes_received,
            const boost::posix_time::ptime &time_packet_sent,
            const std::string &remote_address,
            const uint32_t ttl) const
{
    uint16_t sequence_number = get_icmp_sequence_number();

    GlobalLogger.info() << "Destination " << remote_address
         << " (icmp_seq=" << sequence_number << ") unreachable!"
         << std::endl;
}

std::string IcmpDestinationUnreachableData::to_string() const
{
    if (IcmpData_PingFailReply::get_icmp_data_offset() == -1)
        // not icmp!
        return "[DestUnrchbleData from non-ICMP request]";
    else
    {
        std::stringstream buf;
        buf << "[DestUnrchbleData: ID=" << std::showbase << std::hex
            << IcmpData_PingFailReply::get_icmp_identifier() << ",seq.nr="
            << std::noshowbase << std::dec
            << IcmpData_PingFailReply::get_icmp_sequence_number() << "]";
        return buf.str();
    }
}

// (created using vim -- the world's best text editor)

